@@ -3,7 +3,7 @@ GeoUtils is a python package of raster and vector tools.
 """
 
 from geoutils import spatial_tools  # noqa
-from geoutils import datasets, georaster, geovector, projtools, satimg  # noqa
+from geoutils import examples, georaster, geovector, projtools, satimg  # noqa
 from geoutils.georaster import Raster  # noqa
 from geoutils.geovector import Vector  # noqa
 from geoutils.satimg import SatelliteImage  # noqa
