@@ -7,7 +7,7 @@ Optional dependencies:
 from __future__ import annotations
 
 import warnings
-from typing import Callable
+from typing import Any, Callable
 
 import numpy as np
 import rasterio as rio
@@ -16,7 +16,7 @@ from tqdm import tqdm
 
 import geoutils as gu
 from geoutils.georaster import Raster, RasterType
-from geoutils.georaster.raster import _default_ndv
+from geoutils.georaster.raster import _default_nodata
 from geoutils.misc import resampling_method_from_str
 
 
@@ -88,6 +88,99 @@ def get_valid_extent(array: np.ndarray | np.ma.masked_array) -> tuple[int, ...]:
     return rows_nonzero[0], rows_nonzero[-1], cols_nonzero[0], cols_nonzero[-1]
 
 
+def load_multiple_rasters(
+    raster_paths: list[str], crop: bool = True, ref_grid: int | None = None, **kwargs: Any
+) -> list[RasterType]:
+    """
+    Function to load multiple rasters at once in a memory efficient way.
+    First load metadata only.
+    Optionally, crop all rasters to their intersection (default).
+    Optionally, reproject all rasters to the grid of one raster set as reference (after optional crop).
+    Otherwise, simply load the full rasters.
+
+    :param raster_paths: List of paths to the rasters to be loaded
+    :param crop: if set to True, will only load rasters in the area they intersect
+    :param ref_grid: If set to an integer value, the raster with that index will be considered as the reference
+    and all other rasters will be reprojected on the same grid (after optional crop)
+    :param kwargs: optional arguments to be passed to Raster.reproject, e.g. the resampling method
+
+    :returns: a list of loaded Raster instances
+    """
+    # If ref_grid is provided, need to reproject
+    if isinstance(ref_grid, int):
+        reproject = True
+    # if no ref_grid provided, still need a reference CRS, use first by default
+    elif ref_grid is None:
+        ref_grid = 0
+        reproject = False
+    else:
+        raise ValueError("`ref_grid` must be None or an integer")
+
+    # Need to define a reference CRS for calculating intersection
+    ref_crs = gu.Raster(raster_paths[ref_grid], load_data=False).crs
+
+    # First load all rasters metadata
+    output_rst = []
+    bounds = []
+    for path in raster_paths:
+        # Initialize raster
+        rst = gu.Raster(path, load_data=False)
+        output_rst.append(rst)
+
+        # Get bound in reference CRS
+        bound = rst.get_bounds_projected(ref_crs)
+        bounds.append(bound)
+
+    # Second get the intersection of all raster bounds
+    intersection = gu.projtools.merge_bounds(bounds, "intersection")
+
+    # Optionally, crop the rasters
+    if crop:
+        # Check that intersection is not void
+        if intersection == ():
+            warnings.warn("Intersection is void, returning unloaded rasters.")
+            return output_rst
+
+        for rst in output_rst:
+            # Calculate bounds in rst's CRS
+            # rasterio's default for densify_pts is too low for very large images, set a default of 5000
+            new_bounds = rio.warp.transform_bounds(
+                ref_crs, rst.crs, intersection[0], intersection[1], intersection[2], intersection[3], densify_pts=5000
+            )
+            # Ensure bounds align with the original ones, to avoid resampling at this stage
+            new_bounds = gu.projtools.align_bounds(rst.transform, new_bounds)
+            rst.crop(new_bounds, mode="match_pixel")
+
+    # Optionally, reproject all rasters to the reference grid
+    if reproject:
+
+        ref_rst = output_rst[ref_grid]
+
+        # Set output bounds - intersection if crop is True, otherwise use that of ref_grid
+        if crop:
+            # make sure new bounds align with reference's bounds (to avoid resampling ref)
+            new_bounds = intersection
+            new_bounds = gu.projtools.align_bounds(ref_rst.transform, intersection)
+        else:
+            new_bounds = ref_rst.bounds
+
+        # Reproject all rasters
+        for index, rst in enumerate(output_rst):
+            out_rst = rst.reproject(
+                dst_crs=ref_rst.crs, dst_bounds=new_bounds, dst_res=ref_rst.res, silent=True, **kwargs
+            )
+            if not out_rst.is_loaded:
+                out_rst.load()
+            output_rst[index] = out_rst
+
+    # if no crop or reproject option, simply load the rasters
+    if (not crop) & (not reproject):
+        for rst in output_rst:
+            rst.load()
+
+    return output_rst
+
+
 def merge_bounding_boxes(bounds: list[rio.coords.BoundingBox], resolution: float) -> rio.coords.BoundingBox:
     max_bounds = dict(zip(["left", "right", "top", "bottom"], [np.nan] * 4))
     for bound in bounds:
@@ -164,6 +257,7 @@ height2 and width2 are set based on reference's resolution and the maximum exten
             raster.load()
             raster.is_loaded = False
 
+        nodata = reference_raster.nodata or gu.georaster.raster._default_nodata(reference_raster.data.dtype)
         # Reproject to reference grid
         reprojected_raster = raster.reproject(
             dst_bounds=dst_bounds,
@@ -173,6 +267,7 @@ height2 and width2 are set based on reference's resolution and the maximum exten
             dst_nodata=reference_raster.nodata,
             silent=True,
         )
+        reprojected_raster.set_nodata(nodata)
 
         # Optionally calculate difference
         if diff:
@@ -180,8 +275,8 @@ height2 and width2 are set based on reference's resolution and the maximum exten
             diff_to_ref, _ = get_array_and_mask(diff_to_ref)
             data.append(diff_to_ref)
         else:
-            img_data, _ = get_array_and_mask(reprojected_raster.data.squeeze())
-            data.append(img_data)
+            # img_data, _ = get_array_and_mask(reprojected_raster.data.squeeze())
+            data.append(reprojected_raster.data.squeeze())
 
         # Remove unloaded rasters
         if not raster.is_loaded:
@@ -192,7 +287,7 @@ height2 and width2 are set based on reference's resolution and the maximum exten
     if reference_raster.nodata is not None:
         nodata = reference_raster.nodata
     else:
-        nodata = _default_ndv(data.dtype)
+        nodata = _default_nodata(data.dtype)
     data[np.isnan(data)] = nodata
 
     # Save as gu.Raster - needed as some child classes may not accept multiple bands
@@ -279,7 +374,7 @@ If several algorithms are provided, each result is returned as a separate band.
     if reference_raster.nodata is not None:
         nodata = reference_raster.nodata
     else:
-        nodata = _default_ndv(merged_data.dtype)
+        nodata = _default_nodata(merged_data.dtype)
     merged_data[np.isnan(merged_data)] = nodata
 
     # Save as gu.Raster
